import React, {useState, Suspense, useCallback} from "react";
// import logo from './logo.svg';
import './App.css';

import InputComponent from "./components/InputComponent";
import SonComponent from "./components/SonComponent";
import SonComponent2 from "./components/SonComponent2";
import {Link, NavLink, Route, Routes} from "react-router-dom";
import SonPage from "./pages/SonPage";
import Page3 from "./pages/Page3";

// import store from "./store/msgReducer";
import {connect} from "react-redux"

import {changeMsg} from "./store/tookit";

// 异步组件, 懒加载
const LazyPage = React.lazy(()=> import("./pages/SonPageLazy"))

function App(props) {
    console.log("App props =>", props)
    // console.log("store", store, store.getState())

    // vue3 组合式 API
    // React 必须大写 Pascal 命名 组件
    function FnHello({name = "Fn"}) {
        return <div key={name}>Hello world By {name}</div>
    }

    let com = FnHello({name: "obj"})

    function FnHello2() {
        return React.createElement("div", [], "Hello world By React.createElement")
    }

    // vue2 选项式 API
    class ClassHello extends React.Component {
        // state = {
        //     a: 0
        // }

        constructor(props) {
            super(props);

            this.state = {
                a: 0,
                arr: [0, 1, 2],
                msg: "parent msg"
            }
        }

        subValState = () => {
            this.setState({
                ...this.state,
                a: this.state.a - 1
            })
        }

        addValState() {
            // this.state.a ++
            // this.setState({})
            this.setState((state) => {
                return {
                    ...state,
                    a: ++state.a,
                }
            }, () => {
                console.log("setState callBack, 能获取到修改后的值 => ", this.state.a)
            })

            console.log("setState 之外, 获取不到修改后的值 => ", this.state.a)
        }

        addArr = () => {
            this.setState((state) => {
                state.arr.push(state.arr[state.arr.length - 1] + 1)
                return {
                    ...state,
                    // 解构赋值 使其地址改变 才能触发视图更新
                    // 不直接操作原对象, 要 先拷贝 再操作
                    arr: [...state.arr],
                }
            }, () => {
                console.log("state.arr => ", this.state.arr)
            })
        }

        render() {
            return <div style={{border: '1px solid', width: '80%', overflow: 'auto'}}>
                <div>Hello world By Class</div>
                <div> state => {this.state.a}</div>

                <div style={{display: "flex", justifyContent: "center"}}>
                    <button onClick={this.subValState}>-1</button>
                    <button onClick={this.addValState.bind(this)}>+1</button>
                </div>


                <div>{this.state.arr}</div>
                <div style={{display: "flex", justifyContent: "center"}}>
                    <button onClick={this.addArr}>+1</button>
                </div>
            </div>
        }
    }

    // React 不能直接渲染 js 对象
    let obj = {
        a: 1,
        b: 2
    }

    // 如果是数组则依次遍历
    let arr1 = [
        1,
        2,
        3
    ]

    // 数组渲染
    let arr2 = [
        FnHello({name: "item1"}),
        FnHello({name: "item2"}),
        FnHello({name: "item3"}),
    ]

    function clickHandler2() {
        console.log("按钮2被点击")
    }

    function clickHandler3(arg, e) {
        console.log("按钮3被点击, 参数 " + arg)
        console.log("合成的事件对象", e)
        // 阻止事件冒泡
        e.stopPropagation()
    }

    class IfComponent extends React.PureComponent {
        state = {
            show: true
        }

        show = () => {
            this.setState((state) => ({
                ...state,
                show: true
            }))
        }

        hide = () => {
            this.setState((state) => ({
                ...state,
                show: false
            }))
        }

        render() {
            return <div style={{border: '1px solid', width: '80%', overflow: 'auto'}}>
                <div>条件渲染</div>

                {this.state.show && <div> IfComponent 显示 </div>}

                <div style={{display: 'flex', justifyContent: "center"}}>
                    <button onClick={this.hide}>隐藏</button>
                    <button onClick={this.show}>显示</button>
                </div>
                <button onClick={() => {
                    this.setState((state) => ({
                        ...state,
                        show: !state.show
                    }))
                }}>切换
                </button>
            </div>
        }
    }

    const [msg,setMsg] = useState("parent msg")

    const changeMsg = (newMsg) => {
        setMsg(newMsg)
        return msg;
    }

    // let changeStoreMsg = useCallback(()=>{
    //     store.dispatch({
    //         type: "change",
    //         payload: "修改后的 store msg"
    //     })
    //
    //     console.log("store", store, store.getState())
    // }, [])
    //
    // let changeStoreMsg = useCallback(()=>{
    //     props.dispatch({
    //         type: "change",
    //         payload: "修改后的 redux state msg"
    //     })
    //
    //     console.log("store", props, props.msg)
    // }, [props])

    return (
        <div className="App">
            <div>
                <span>跳转至: </span>
                {/*NavLink 会在当前路由匹配时 加一个 class 名为 active 的样式*/}
                <NavLink to={"/"}> 首页 </NavLink>
                <NavLink to={"router-page1"}> 页面1 </NavLink>
                <NavLink to={"router-page2"}> 页面2 </NavLink>
                <NavLink to={"router-page3/son-page"}> 页面3-子页面 </NavLink>
                <NavLink to={"router-page3/son-page2?page=2"}> 页面3-子页面2 </NavLink>
            </div>

            <div>
                <span>跳转至: </span>
                <Link to={"/"}> 首页 </Link>
                <Link to={"router-page1"}> 页面1 </Link>
                <Link to={"router-page2"}> 页面2 </Link>
            </div>

            {/*过渡效果*/}
            <Suspense fallback={<div>...加载中...</div>}>
                {/* react router v5 和 v6 有较大区别*/}
                {/* v5 中为 Switch*/}
                <Routes>
                    {/* v6 为 element 或 Component */}
                    <Route path="/router-page1" element={<div>Router Page 1</div>}></Route>
                    {/* v5 只有 component */}
                    <Route path="/router-page2" Component={() => <div>Router Page 2</div>}></Route>

                    <Route path="/router-page3" element={<Page3/>}>
                        <Route path={"son-page"} element={<SonPage/>}></Route>

                        <Route path={"son-page2"} Component={LazyPage}></Route>
                    </Route>
                </Routes>
            </Suspense>

            {/*class 类名必须为 className, 内容只能是字符串*/}
            <header className="App-header">
                {/*<img src={logo} className="App-logo" alt="logo" />*/}

                {/*数组渲染*/}
                {[
                    // key 必须唯一
                    <FnHello key={1}/>,
                    <FnHello2 key={2}/>
                ]}

                <ClassHello/>

                {com}

                {JSON.stringify(obj)}

                <br/>

                {arr1}

                {arr2}


                {/*style 内联样式 必须写成对象的形式 */}
                <div style={{display: 'flex'}}>
                    {/*事件绑定*/}
                    <button onClick={() => {
                        console.log("按钮1被点击")
                    }}>按钮1
                    </button>

                    <button onClick={clickHandler2}>按钮2</button>
                </div>
                <div>
                    <button onClick={clickHandler3.bind(this, "按钮3")}>按钮3(bind)</button>
                    <button onClick={(e) => clickHandler3("按钮3", e)}>按钮3</button>
                </div>

                <IfComponent/>

                <InputComponent/>

                <div style={{
                    border: '1px solid',
                    width: '80%',
                    overflow: 'auto',
                    display: 'flex',
                    flexDirection: 'column',
                    alignItems: 'center',
                    justifyContent: 'center'
                }}>
                    父组件
                    <SonComponent a={1} b={2} msg={"App Msg"}
                                  slot2={<div>SonComponent 插槽 slot2 (具名插槽)</div>}
                                  scopeslot={(scope) => {
                                      return <div>{scope}</div>
                                  }}>
                        <div>
                            SonComponent 插槽 slot1
                        </div>
                    </SonComponent>

                    <SonComponent2 msg={msg} changeMsg={changeMsg}></SonComponent2>
                </div>

                <div>
                    Redux

                    <div>
                        {props.msg}
                    </div>
                    <div>
                        <button onClick={props.changeStoreMsg}>修改 redux state msg</button>
                    </div>
                </div>
            </header>
        </div>
    );
}

// connect
// 第一个参数 映射 state, 把需要使用的 state 映射到组件的 props
// 第二个参数 为方法映射, 往 props 传入哪些方法
export default connect((state) => {
        console.log("connect state", state)
        return {
            msg: state.msgReducer.msg
        }
    }, (dispatch) => ({
        changeStoreMsg: () => {
            // redux 原生
            // dispatch({
            //     type: "changeMsg",
            //     payload: "修改后的 redux state msg"
            // })

            // toolkit 方式 1
            // dispatch({
            //     // toolkit 格式 => 切片名称/方法名
            //     type: "msgSlice/changeMsg",
            //     payload: "修改后的 redux toolkit state msg"
            // })

            // toolkit 方式 2
            dispatch(changeMsg("修改后的 redux toolkit state msg"))
        }
    })
)(App);
